import csv
import sys
from ..objects.nutrient_dict import *

def report(meal):
    report = []
    for i in range(0, len(meal.nutrients)):
        name = meal.nutrients[i]["name"]
        rda = nutrient_dict[i]["rda"]
        limit = nutrient_dict[i]["limit"]
        value = meal.nutrients[i]["value"]
        state = ""
        if rda == None:
            rda = 0
        if limit == None:
            limit = sys.maxsize
        if value < rda:
            state = "deficient"
        elif value > limit:
            state = "excessive"
        else:
            state = "satisfactory"
        if limit == sys.maxsize:
            limit = None
        report.append({"name":name, "rda":rda, "limit":limit, "value":value, "state":state})
    return report

def export_report(meal, path):
    csvfile = open(path, "w",newline='')
    writer = csv.writer(csvfile, delimiter=',')
    # Write profile information
    writer.writerow(['TDEE',None,tdee])
    writer.writerow(['Carb Ratio',carb_p,carb_p*tdee])
    writer.writerow(['Protein Ratio',protein_p,protein_p*tdee])
    writer.writerow(['Fat Ratio',fat_p,fat_p*tdee])
    writer.writerow([''])
    # Write nutritional information
    writer.writerow(['Nutrient', 'RDA', 'Limit'])
    for i in range(0, len(meal.nutrients)):
        name = meal.nutrients[i]["name"]
        row = [name]
        if nutrient_dict[i]["rda"] == None:
            row.append("None")
        else:
            row.append(nutrient_dict[i]["rda"])
        if nutrient_dict[i]["limit"] == None:
            row.append("None")
        else:
            row.append(nutrient_dict[i]["limit"])
        row.append(meal.nutrients[i]["value"])
        i += 1
        writer.writerow(row)


from ..client.main import Client
from ..client.dict_parse import food_parse
from ..objects.nutrient_dict import nutrient_dict
from ..objects.food import Meal

def foods(id_value_dict, client):
    food_obj = client.food_query(id_value_dict.keys())
    # Return list of food objects in context of nutrients being tracked
    food_obj = food_parse(food_obj, nutrient_dict, list(id_value_dict.values()))
    return food_obj

def meal(foods):
    return Meal(foods)


from ..client.main import Client
from ..client.dict_parse import search_parse
import operator

def get_results(query, client):
    search_obj = client.search_query(query)
    search_obj = search_parse(search_obj)
    return search_obj

def print_results(search_obj, max_entries=None):
    print("="*112)
    print("Search results for \'{}\' on USDA Standard Reference Database".format(search_obj["search_term"]))
    print("="*112)
    if max_entries == None:
        max_entries = len(search_obj["items"])
    if max_entries < len(search_obj["items"]):
        search_obj["items"] = search_obj["items"][:max_entries]
    search_obj["items"].sort(key=operator.itemgetter("group"))
    print("{name:<72} {group:^30} {id:>8}".format(name="Name",group="Group",id="ID"))
    for item in search_obj["items"]:
        if len(item["name"]) > 70:
            item["name"] = item["name"][:70] + ".."
        if len(item["group"]) > 28:
            item["group"] = item["group"][:28] + ".."
        print("{name:<72} {group:^30} {id:>8}".format(name=item["name"],group=item["group"],id=item["ndbno"]))
    print("="*112)


import requests
import json

class Client:
    url = 'https://api.nal.usda.gov/usda/ndb'

    def __init__(self, key):
        '''
        A Client instance must be initialized with a key from
        data.gov. This is free to obtain, and you can request one
        here: https://api.data.gov/signup/
        '''
        self.key = key

    def call(self, params, url_suffix):
        ''' target_url could be:
        https://api.nal.usda.gov/usda/ndb/V2/reports
        https://api.nal.usda.gov/usda/ndb/search
        depending on which service of the api is being used
        '''
        target_url = self.url + url_suffix 
        # add the key to the API call
        params.update(dict(api_key=self.key))
        response = json.loads(requests.get(url=target_url, params=params).text)
        return response

    def search_query(self, name):
        params = dict(
            q=name,
            ds='Standard Reference',
            format='json'
        )
        return self.call(params, '/search')
    
    def food_query(self, ids):
        # allow for either a single id (ndbno) query, or a list of queries
        if type(ids) == list:
            if len(ids) > 25:
                raise Exception("Too many Food ID arguments. API limits it to 25.")
        params = dict(ndbno=ids)
        params.update(dict(type='f', format='json'))
        return_obj = self.call(params, '/V2/reports')
        for food in return_obj["foods"]:
            if 'error' in food.keys():
                raise Exception(food["error"])
        return return_obj


import operator
from ..objects.food import Food

def search_parse(search_results):
    # Store the search term that was used to produce these results
    search_term = search_results["list"]["q"]
    # Store a list of dictionary items for each result of the search
    items = []
    for item in search_results["list"]["item"]:
        # Remove extraneous pieces of data
        del item["ds"]; del item["manu"]; del item["offset"]
        items.append(item)
    return dict(search_term=search_term, items=items)

def food_parse(food_results, nutrient_dict, values):
    # Return a simplified version of the json file returned from the USDA API
    # This deletes extraneous pieces of information, including nutrients that are
    # not tracked. It also exchanges nutrient names for their more common names, or "nicknames"
    food_arr = []
    tracked_nutrients = []
    nutrient_nicknames = []
    for nutrient in nutrient_dict:
        if "nickname" in nutrient.keys():
            nutrient_nicknames.append(nutrient["nickname"])
        else:
            nutrient_nicknames.append(None)
        tracked_nutrients.append(nutrient["nutrient_id"])
    food_results = food_results["foods"]
    # Remove extraneous pieces of data in the food description
    to_del = {'food':["sr", "type", "sources", "footnotes", "langual"], 
              'desc':["sd", "sn", "cn", "manu", "nf", "cf", "ff", "pf", "r", "rd", "ru", "ds"],
              # This current implementation deletes the measurement data from the database,
              # this could be changed later to provide richer data and easy UX tools (select measure)
              'nutrients':["derivation", "sourcecode", "dp", "se", "measures"]}
    # Iterate through each food and remove extra information, and simplify names
    f = 0
    for food in food_results:
        for del_item in to_del["food"]:
            del food["food"][del_item]
        for del_item in to_del["desc"]:
            del food["food"]["desc"][del_item]
        # sort nutrients by id if not already
        n_list = food["food"]["nutrients"]
        n_list.sort(key=operator.itemgetter("nutrient_id"))
        # end sort
        n = 0
        for nutrient in food["food"]["nutrients"]:
            if n == len(tracked_nutrients):
                break
            # check if this is a nutrient we should record
            if nutrient["nutrient_id"] == tracked_nutrients[n]:
                potential_name = nutrient_nicknames[n]
                if potential_name != None:
                    nutrient["name"] = potential_name
                for del_item in to_del["nutrients"]:
                    del nutrient[del_item]
                n += 1
            # check if the food doesn't contain a tracked nutrient
            while n < len(tracked_nutrients) and nutrient["nutrient_id"] > tracked_nutrients[n]:
                to_insert = nutrient_dict[n]
                if "nickname" in to_insert.keys():
                    to_insert["name"] = to_insert["nickname"]
                    del to_insert["nickname"]
                to_insert.update(value=0)
                food["food"]["nutrients"].insert(n,to_insert)
                n += 1
        while n < len(tracked_nutrients) and food["food"]["nutrients"][-1]["nutrient_id"] < tracked_nutrients[-1]:
            to_insert = nutrient_dict[n]
            if "nickname" in to_insert.keys():
                to_insert["name"] = to_insert["nickname"]
                del to_insert["nickname"]
            to_insert.update(value=0)
            food["food"]["nutrients"].insert(n,to_insert)
            n += 1
        n = 0
        n_to_del = []
        for nutrient in food["food"]["nutrients"]:
            # check if this is a nutrient we should delete
            if nutrient["nutrient_id"] not in tracked_nutrients:
                n_to_del.append(n)
            n += 1
        offset = 0
        for del_n in n_to_del:
            del food["food"]["nutrients"][del_n - offset]
            offset += 1
        # sort nutrients by id if not already
        n_list = food["food"]["nutrients"]
        n_list.sort(key=operator.itemgetter("nutrient_id"))
        # end sort
        for nutrient in food["food"]["nutrients"]:
            nutrient["value"] = nutrient["value"] * (values[f]/100)
        f += 1
        food_arr.append(Food(food))
    return food_arr


import copy

class Food:
    def __init__(self, data):
        self.desc = data["food"]["desc"]
        self.nutrients = data["food"]["nutrients"]

class Meal:
    def __init__(self, foods):
        self.foods = foods
        self.nutrients = []
        for nutrient in foods[0].nutrients:
            to_app = nutrient.copy()
            to_app["value"] = 0
            self.nutrients.append(to_app)
        for food in foods:
            n = 0
            for nutrient in food.nutrients:
                self.nutrients[n]["value"] += nutrient["value"]
                n += 1
        for n in range(0, len(self.nutrients)):
            self.nutrients[n]["value"] = round(self.nutrients[n]["value"], 2)



import json
import os
import codecs

dir_path = os.path.dirname(os.path.realpath(__file__))
nutrient_file = codecs.open("{}/nutrient_ids.json".format(dir_path), encoding="utf-8").read()
nutrient_dict = json.loads(nutrient_file)

# PROFILE INFORMATION
tdee = 2000 #kcal per day
# What percent of daily caloric intake should each macro take?
protein_p = 0.25
carb_p = 0.50
fat_p = 0.25
# Sugar should be no more than 10% of total calories
# US dietary guidelines: https://health.gov/dietaryguidelines/2015/guidelines/executive-summary/
sugar_p = 0.10

# Assign RDAs, some dependent on gender or tdee
for item in nutrient_dict:
    id = item["nutrient_id"]
    # PROXIMATES
    if id == 203: # Protein, g
        item.update(rda=(tdee/4)*protein_p)
    if id == 204: # Fat, g
        item.update(rda=(tdee/9)*fat_p)
    if id == 205: # Carbs, g
        item.update(rda=(tdee/4)*carb_p)
    if id == 207: # Ash, g
        item.update(rda=None)
    if id == 208: # Calories, kcal
        item.update(rda=tdee)
    if id == 221: # Alcohol, g note: 7 calories per gram
        item.update(rda=None)
    if id == 255: # Water, g
        item.update(rda=2000)
    # OTHER
    if id == 262: # Caffeine, mg
        item.update(rda=None)
        item.update(limit=400)
    if id == 263: # Theobromine, mg
        item.update(rda=None)
        item.update(limit=300)
    # PROXIMATES
    if id == 269: # Sugar, g
        item.update(rda=None)
        item.update(limit=(tdee/4)*sugar_p)
    if id == 291: # Fiber, g note: 14 grams for every 1000 calories
        item.update(rda=tdee*0.014)
    # MINERALS
    if id == 301: # Calcium, mg
        item.update(rda=1000)
        item.update(limit=2500)
    if id == 303: # Iron, mg
        item.update(rda=8)
        item.update(limit=45)
    if id == 304: # Magnesium, mg
        item.update(rda=300)
        item.update(limit=700)
    if id == 305: # Phosphorus, mg
        item.update(rda=700)
        item.update(limit=4000)
    if id == 306: # Potassium, mg
        item.update(rda=1400)
        item.update(limit=6000)
    if id == 307: # Sodium, mg
        item.update(rda=1000)
        item.update(limit=2300)
    if id == 309: # Zinc, mg
        item.update(rda=12)
        item.update(limit=100)
    if id == 312: # Copper, mg
        item.update(rda=0.9)
        item.update(limit=10)
    if id == 313: # Fluoride, ug
        item.update(rda=400)
        item.update(limit=10000)
    if id == 315: # Manganese, mg
        item.update(rda=1.8)
    if id == 317: # Selenium, ug
        item.update(rda=70)
        item.update(limit=400)
    # VITAMINS
    if id == 318: # Vitamin A, IU
        item.update(rda=900)
        item.update(limit=20000)
    if id == 323: # Vitamin E, mg
        item.update(rda=15)
        item.update(limit=1000)
    if id == 324: # Vitamin D, IU
        item.update(rda=1000)
        item.update(limit=8000)
    if id == 401: # Vitamin C, mg
        item.update(rda=90)
        item.update(limit=2000)
    if id == 404: # Vitamin B-1, mg
        item.update(rda=1.2)
    if id == 405: # Vitamin B-2, mg
        item.update(rda=1.3)
    if id == 406: # Vitamin B-3, mg
        item.update(rda=16)
    if id == 410: # Vitamin B-5, mg
        item.update(rda=4)
    if id == 415: # Vitamin B-6, mg
        item.update(rda=1.3)
        item.update(limit=100)
    if id == 417: # Vitamin B-9, ug
        item.update(rda=400)
        item.update(limit=1000)
    if id == 418: # Vitamin B-12, mg
        item.update(rda=2.4)
    if id == 421: # Choline, mg
        item.update(rda=550)
        item.update(limit=3500)
    if id == 430: # Vitamin K, ug
        item.update(rda=120)
    # LIPIDS
    if id == 601: # Cholesterol, mg
        item.update(rda=None)
        item.update(limit=300)
    if id == 605: # Trans Fat, g
        item.update(rda=None)
        # avoid more than 5% of fat calories from trans fat
        item.update(limit=(tdee/9)*(fat_p*0.05))
    if id == 606: # Saturated Fat, g
        item.update(rda=None)
        item.update(limit=(tdee/9)*(fat_p*0.3))
    if id == 621: # DHA, g
        item.update(rda=0.5)
    if id == 629: # EPA, g
        item.update(rda=0.5)
    if id == 645: # Monounsaturated Fat, g
        item.update(rda=(tdee/9)*(fat_p*.40))
    if id == 646: # Polyunsaturated Fat, g
        item.update(rda=(tdee/9)*(fat_p*.30))
    if id == 851: # ALA, g
        item.update(rda=0.6)
    if "limit" not in item.keys():
        item.update(limit=None)

# Round values to avoid long decimals
for item in nutrient_dict:
    if item["rda"] != None:
        item["rda"] = round(item["rda"], 2)
    if item["limit"] != None:
        item["limit"] = round(item["limit"], 2)